package jp.programmingmat.www.gbtkgl10;

import javax.microedition.khronos.opengles.GL10;

import android.opengl.GLU;

/**
 * カメラ（視点・視界情報）保持クラス。
 */
public class GbtkGL10Camera {

	// 視点（現在位置）
	private GbtkGL10Vector3f mPosition;

	// 視線ベクトル
	private GbtkGL10Vector3f mLookVector;

	// 注視点（LookATに指定）
	private GbtkGL10Vector3f mLook;

	// カメラ姿勢（上向き）ベクトル
	private GbtkGL10Vector3f mUpVector;
	
	private float mNear, mFar;
	private float mAspect;
	private float mFovy;

	public GbtkGL10Camera() {

		mAspect = 1.0f;
		mFovy = 75;

		mNear = 0.1f;
		mFar = 1000.0f;

		mPosition = new GbtkGL10Vector3f();
		mLookVector = new GbtkGL10Vector3f();
		mLook = new GbtkGL10Vector3f();
		mUpVector = new GbtkGL10Vector3f();

		setPosition(0, 0, 0);
		setLookVector(0, 0, -100);
		setUpVector(0.0f, 1.0f, 0.0f);
		
	}

	/**
	 * カメラが配置されている現在位置を取得。
	 * 
	 * @return カメラの現在位置
	 */
	public GbtkGL10Vector3f getPosition() {
		return mPosition;
	}
	
	/**
	 * カメラを配置する位置（視点）を設定。
	 * @param x カメラを配置するX座標
	 * @param y カメラを配置するY座標
	 * @param z カメラを配置するZ座標
	 */
	public void setPosition(float x, float y, float z) {

		mPosition.set(x, y, z);

		// 更新された現在位置（視点）に注視方向ベクトルを加えてLookAtの注視点を再計算
		mLook.set(mPosition.get(0) + mLookVector.get(0), mPosition.get(1) + mLookVector.get(1), mPosition.get(2) + mLookVector.get(2));
		
	}

	/**
	 * カメラの注視方向を設定。
	 * @param x 注視方向ベクトルのX成分
	 * @param y 注視方向ベクトルのY成分
	 * @param z 注視方向ベクトルのZ成分
	 */
	public void setLookVector(float x, float y, float z) {

		mLookVector.set(x, y, z);

		// 現在位置（視点）に更新された注視方向ベクトルを加えてLookAtの注視点を再計算
		mLook.set(mPosition.get(0) + mLookVector.get(0), mPosition.get(1) + mLookVector.get(1), mPosition.get(2) + mLookVector.get(2));
		
	}

	/**
	 * カメラの角度（カメラ上側の向きベクトル）を設定。
	 * @param x 方向ベクトルのX座標
	 * @param y 方向ベクトルのY座標
	 * @param z 方向ベクトルのZ座標
	 */
	public void setUpVector(float x, float y, float z) {
		mUpVector.set(x, y, z);
	}

	/**
	 * アスペクト（縦横比）を設定。
	 * 
	 * @param aspect 横方向サイズ／縦方向サイズの実数値
	 */
	public void setAspect(float aspect) {
		mAspect = aspect;
	}

	/**
	 * カメラ視界の近方限界を設定。
	 * @param near 近方限界として設定するカメラからの距離
	 */
	public void setNear(float near) {
		mNear = near;
	}

	/**
	 * カメラ視界の遠方限界を設定。
	 * @param far 遠方限界として設定するカメラからの距離
	 */
	public void setFar(float far) {
		mFar = far;
	}

	/**
	 * カメラの縦方向視野（画角）を設定。
	 * @param fovy Y方向の画角
	 */
	public void setFovy(float fovy) {
		mFovy = fovy;
	}

	/**
	 * カメラが保持している視界情報を指定のWorldが描画対象とするOpenGL ESシステムに設定する。
	 * @param world カメラの視界情報を設定するworldオブジェクト
	 */
	public void setCamera(GbtkGL10World world) {

		GL10 gl = world.getGLInterface();
		
    	gl.glMatrixMode(GL10.GL_PROJECTION);
    	gl.glLoadIdentity();

        GLU.gluPerspective(gl, mFovy, mAspect, mNear, mFar);
		GLU.gluLookAt(gl, mPosition.get(0), mPosition.get(1), mPosition.get(2), mLook.get(0), mLook.get(1), mLook.get(2), mUpVector.get(0), mUpVector.get(1), mUpVector.get(2));

	}

}
