package jp.programmingmat.www.gbtkgl10;

import android.util.FloatMath;

/**
 * Geometryを構成する基本頂点（図形）データ。
 * 単一のOpenGL ES描画コマンドで描画可能な図形の頂点データを保持します。頂点、法線、テクスチャの各配列を保持するので、頂点数の多い図形を多数扱う場合はメモリ不足に注意してください。
 * @see GbtkGL10Geometry
 */
public class GbtkGL10Element {

	private float[] mVertexArray;
	private float[] mNormalArray;
	private float[] mTexturePointArray;
	
	private int mDrawVertexOffset;
	private int mVertexNum;

	private int mDrawType;

	public static final int DRAWTYPE_TRIANGLES = 1;
	public static final int DRAWTYPE_TRIANGLE_STRIP = 2;
	public static final int DRAWTYPE_TRIANGLE_FAN = 3;

	public GbtkGL10Element() {

	}

	/**
	 * 図形（全頂点）を移動します。
	 * 描画時の位置ではなく、頂点そのものを移動するので注意してください。
	 * @param dx X方向の移動量
	 * @param dy Y方向の移動量
	 * @param dz Z方向の移動量
	 */
	public void translate(float dx, float dy, float dz) {

		if (mVertexArray == null) {
			return;
		}
		
		for (int i = 0;i < mVertexNum;i++) {
			
			mVertexArray[i * 3] += dx;
			mVertexArray[i * 3 + 1] += dy;
			mVertexArray[i * 3 + 2] += dz;
			
		}
			
	}
	
	/**
	 * 描画時のコマンド（OpenGL ESに指定するパラメータ）種別を取得します。
	 * システムの描画時に呼ばれる関数です。
	 * @return 描画時のコマンド種別
	 */
	public int getDrawType() {
		return mDrawType;
	}
	
	/**
	 * 描画時のコマンド（OpenGL ESに指定するパラメータ）を設定します。
	 * 構築関数を使用してオブジェクトを生成する時に呼び出される関数です。
	 * @param type コマンド種別
	 */
	public void setDrawType (int type) {
		mDrawType = type;
	}

	/**
	 * Geometry内における頂点オフセットを取得します。
	 * 描画時に呼び出される関数です。
	 * @return Geometry内における頂点オフセット
	 */
	public int getDrawVertexOffset() {
		return mDrawVertexOffset;
	}

	/**
	 * Geometry内における頂点オフセットを設定します。
	 * Geometryなど描画単位の生成時に呼び出される関数です。
	 * @param offset Geometry内における頂点オフセット
	 */
	public void setDrawVertexOffset(int offset) {
		mDrawVertexOffset = offset;
	}

	/**
	 * 頂点数を取得します。
	 * @return 頂点数
	 */
	public int getVertexNum() {
		return mVertexNum;
	}

	/**
	 * OpenGL ESの頂点データを格納する配列を取得します。
	 * @return 頂点データ配列
	 */
	public float[] getVertexArray() {
		return mVertexArray;
	}
	
	/**
	 * OpenGL ESの頂点データを格納する配列を設定します。
	 * @param vertexes 頂点データ配列
	 */
	public void setVertexArray(float[] vertexes) {

		if (vertexes == null) {

			if (mVertexArray != null) {
				mVertexArray = null;
			}

			return;
			
		}

		mVertexArray = vertexes;
        mVertexNum = vertexes.length / 3;
		
		return;
		
	}

	/**
	 * OpenGL ESの法線データを格納する配列を取得します。
	 * @return 法線データ配列
	 */
	public float[] getNormalArray() {
		return mNormalArray;
	}
	
	/**
	 * OpenGL ESの法線データを格納する配列を設定します。
	 * @param normals 法線データ配列
	 */
	public void setNormalArray(float[] normals) {

		if (normals == null) {

			if (mNormalArray != null) {
				mNormalArray = null;
			}

			return;
			
		}

		mNormalArray = normals;
		
		return;
		
	}

	/**
	 * OpenGL ESのテクスチャ座標データを格納する配列を取得します。
	 * @return テクスチャ座標データ配列
	 */
	public float[] getTexturePointArray() {
		return mTexturePointArray;
	}

	/**
	 * OpenGL ESのテクスチャ座標データを格納する配列を設定します。
	 * @param texturePointArray テクスチャ座標データ配列
	 */
	public void setTexturePointArray(float[] texturePointArray) {
		mTexturePointArray = texturePointArray;
	}

	/**
	 * XY平面に沿って平面elementを作成します。
	 * XY平面に平行な平面（長方形）をあらわすelementを作成します。
	 * @param xSize X方向の大きさ
	 * @param ySize Y方向の大きさ
	 * @param normalZ 平面の向き（Z方向の法線）
	 * @return 作成された平面element
	 */
	public static GbtkGL10Element createXYPlaneElement(float xSize, float ySize, float normalZ) {
		return createXYPlaneElement(xSize, ySize, normalZ, 0, 0, 0);
	}
	
	/**
	 * XY平面に沿って平面elementを作成します。
	 * 指定の位置を中心とするXY平面に平行な平面（長方形）をあらわすelementを作成します。
	 * @param xSize X方向の大きさ
	 * @param ySize Y方向の大きさ
	 * @param normalZ 平面の向き（Z方向の法線）
	 * @param x 平面の中心X座標
	 * @param y 平面の中心Y座標
	 * @param z 平面の中心Z座標
	 * @return 作成された平面element
	 */
	public static GbtkGL10Element createXYPlaneElement(float xSize, float ySize, float normalZ, float x, float y, float z) {

		float xx = xSize / 2.0f;
		float yy = ySize / 2.0f;
		
		float[] vertexList = new float[]{-xx + x, yy + y, z , -xx + x, -yy + y, z, xx + x, yy + y, z, xx + x, -yy + y, z};
		float[] normalList = new float[] {0, 0, normalZ, 0, 0, normalZ, 0, 0, normalZ, 0, 0, normalZ};
		float[] textureArray = new float[] {0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f, 1.0f};

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLE_STRIP);
		
		return elm;
		
	}
	
	/**
	 * XZ平面に沿って平面elementを作成します。
	 * XZ平面に平行な平面（長方形）をあらわすelementを作成します。
	 * @param xSize X方向の大きさ
	 * @param zSize Z方向の大きさ
	 * @param normalY 平面の向き（Y方向の法線）
	 * @return 作成された平面element
	 */
	public static GbtkGL10Element createXZPlaneElement(float xSize, float zSize, float normalY) {
		return createXZPlaneElement(xSize, zSize, normalY, 0, 0, 0);
	}
	
	/**
	 * XZ平面に沿って平面elementを作成します。
	 * 指定の位置を中心とするXZ平面に平行な平面（長方形）をあらわすelementを作成します。
	 * @param xSize X方向の大きさ
	 * @param zSize Z方向の大きさ
	 * @param normalY 平面の向き（Y方向の法線）
	 * @param x 平面の中心X座標
	 * @param y 面の中心Y座標
	 * @param z 平面の中心Z座標
	 * @return 作成された平面element
	 */
	public static GbtkGL10Element createXZPlaneElement(float xSize, float zSize, float normalY, float x, float y, float z) {
		
		float xx = xSize / 2.0f;
		float zz = zSize / 2.0f;
		
		float[] vertexList = new float[] {-xx + x, y, zz + z, -xx + x, y, -zz + z, xx + x, y, zz + z, xx + x, y, -zz + z};
		float[] normalList = new float[] {0, normalY, 0, 0, normalY, 0, 0, normalY, 0, 0, normalY, 0};
		float[] textureArray = new float[] {0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f, 0.0f};
		
		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);
		
		elm.setDrawType(DRAWTYPE_TRIANGLE_STRIP);
		
		return elm;
		
	}

	/**
	 * YZ平面に沿って平面elementを作成します。
	 * YZ平面に平行な平面（長方形）をあらわすelementを作成します。
	 * @param ySize Y方向の大きさ
	 * @param zSize Z方向の大きさ
	 * @param normalX 平面の向き（X方向の法線）
	 * @return 作成された平面element
	 */
	public static GbtkGL10Element createYZPlaneElement(float ySize, float zSize, float normalX) {
		return createYZPlaneElement(ySize, zSize, normalX, 0, 0, 0);
	}
	
	/**
	 * YZ平面に沿って平面elementを作成します。
	 * 指定の位置を中心とするYZ平面に平行な平面（長方形）をあらわすelementを作成します。
	 * @param ySize Y方向の大きさ
	 * @param zSize Z方向の大きさ
	 * @param normalX 平面の向き（X方向の法線）
	 * @param x 平面の中心X座標
	 * @param y 平面の中心Y座標
	 * @param z 平面の中心Z座標
	 * @return 作成された平面element
	 */
	public static GbtkGL10Element createYZPlaneElement(float ySize, float zSize, float normalX, float x, float y, float z) {

		float yy = ySize / 2.0f;
		float zz = zSize / 2.0f;
		
		float[] vertexList = new float[] {x, -yy + y, zz + z, x, -yy + y, -zz + z, x, yy + y, zz + z, x, yy + y, -zz + z};
		float[] normalList = new float[] {normalX, 0, 0, normalX, 0, 0, normalX, 0, 0, normalX, 0, 0};
		float[] textureArray = new float[] {0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f, 0.0f};

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLE_STRIP);
		
		return elm;
		
	}

	/**
	 * XY平面に沿って円（多角形）elementを作成します。
	 * XY平面に平行な円（厚みを持たない円平面）をあらわすelementを作成します。
	 * @param r 円の半径
	 * @param div 頂点データ作成時の円分割数
	 * @param normalZ 平面の向き（Z方向の法線）
	 * @return 作成された円element
	 */
	public static GbtkGL10Element createXYCircleElement(float r, int div, float normalZ) {

		float[] vertexList = new float[((div + 2) * 3)];
		float[] normalList = new float[((div + 2) * 3)];
		float[] textureArray = new float[((div + 2) * 2)];

		vertexList[0] = 0;
		vertexList[1] = 0;
		vertexList[2] = 0;

		normalList[0] = 0;
		normalList[1] = 0;
		normalList[2] = normalZ;

		textureArray[0] = 0.5f;
		textureArray[1] = 0.5f;
		
		float pi = (float)Math.PI;

		// div角形として頂点を作成
		for (int i = 0;i <= div;i++) {
			
			// 円周上のx/y座標を算出
			float x = FloatMath.cos((pi * 2.0f) * ((float)i / (float)div));
			float y = FloatMath.sin((pi * 2.0f) * ((float)i / (float)div));

			vertexList[(i + 1) * 3] = r * x;
			vertexList[(i + 1) * 3 + 1] = r * y;
			vertexList[(i + 1) * 3 + 2] = 0;

			normalList[(i + 1) * 3] = 0;
			normalList[(i + 1) * 3 + 1] = 0;
			normalList[(i + 1) * 3 + 2] = normalZ;

			textureArray[(i + 1) * 2] = 0.5f + (x / 2);
			textureArray[1 + (i + 1) * 2] = 0.5f + (y / 2);
			
		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);
		
		elm.setDrawType(DRAWTYPE_TRIANGLE_FAN);
		
		return elm;
		
	}

	/**
	 * XZ平面に沿って円elementを作成します。
	 * XZ平面に平行な円（厚みを持たない円平面）をあらわすelementを作成します。
	 * @param r 円の半径
	 * @param div 頂点データ作成時の円分割数
	 * @param normalY 平面の向き（Y方向の法線）
	 * @return 作成された円element
	 */
	public static GbtkGL10Element createXZCircleElement(float r, int div, float normalY) {

		float[] vertexList = new float[((div + 2) * 3)];
		float[] normalList = new float[((div + 2) * 3)];
		float[] textureArray = new float[((div + 2) * 2)];

		vertexList[0] = 0;
		vertexList[1] = 0;
		vertexList[2] = 0;

		normalList[0] = 0;
		normalList[1] = normalY;
		normalList[2] = 0;

		textureArray[0] = 0.5f;
		textureArray[1] = 0.5f;
		
		float pi = (float)Math.PI;
		
		// div角形として頂点を作成
		for (int i = 0;i <= div;i++) {
			
			float x = FloatMath.cos((pi * 2.0f) * ((float)i / (float)div));
			float z = FloatMath.sin((pi * 2.0f) * ((float)i / (float)div));

			vertexList[(i + 1) * 3] = r * x;
			vertexList[(i + 1) * 3 + 1] = 0;
			vertexList[(i + 1) * 3 + 2] = r * z;

			normalList[(i + 1) * 3] = 0;
			normalList[(i + 1) * 3 + 1] = normalY;
			normalList[(i + 1) * 3 + 2] = 0;

			textureArray[(i + 1) * 2] = 0.5f + (x / 2);
			textureArray[1 + (i + 1) * 2] = 0.5f + (z / 2);
			
		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLE_FAN);
		
		return elm;
		
	}

	/**
	 * YZ平面に沿って円elementを作成します。
	 * YZ平面に平行な円（厚みを持たない円平面）をあらわすelementを作成します。
	 * @param r 円の半径
	 * @param div 頂点データ作成時の円分割数
	 * @param normalX 平面の向き（X方向の法線）
	 * @return 作成された円element
	 */
	public static GbtkGL10Element createYZCircleElement(float r, int div, float normalX) {

		float[] vertexList = new float[((div + 2) * 3)];
		float[] normalList = new float[((div + 2) * 3)];
		float[] textureArray = new float[((div + 2) * 2)];

		vertexList[0] = 0;
		vertexList[1] = 0;
		vertexList[2] = 0;

		normalList[0] = normalX;
		normalList[1] = 0;
		normalList[2] = 0;

		textureArray[0] = 0.5f;
		textureArray[1] = 0.5f;
		
		float pi = (float)Math.PI;
		
		// div角形として頂点を作成
		for (int i = 0;i <= div;i++) {
			
			float y = FloatMath.cos((pi * 2.0f) * ((float)i / (float)div));
			float z = FloatMath.sin((pi * 2.0f) * ((float)i / (float)div));

			vertexList[(i + 1) * 3] = 0;
			vertexList[(i + 1) * 3 + 1] = r * y;
			vertexList[(i + 1) * 3 + 2] = r * z;

			normalList[(i + 1) * 3] = normalX;
			normalList[(i + 1) * 3 + 1] = 0;
			normalList[(i + 1) * 3 + 2] = 0;

			textureArray[(i + 1) * 2] = 0.5f + (y / 2);
			textureArray[1 + (i + 1) * 2] = 0.5f + (z / 2);
			
		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLE_FAN);
		
		return elm;
		
	}

	/**
	 * XY平面に沿った円を底とする円柱（中空）elementを作成します。
	 * XY平面に沿った円を底とする円柱（中空）をあらわすelementを作成します。
	 * @param r 底の半径
	 * @param length 円柱の長さ
	 * @param div 円の分割数
	 * @param normalR 円柱表面の向き（中心から見た法線。プラスなら表面が表、マイナスなら裏面が表）
	 * @return 作成された円柱element
	 */
	public static GbtkGL10Element createXYCylinderElement(float r, float length, int div, float normalR) {

		float[] vertexList = new float[div * 3 * 4];
		float[] normalList = new float[div * 3 * 4];
		float[] textureArray = new float[div * 4 * 2];

		float pi = (float)Math.PI;

		float z1 = -(length / 2.0f);
		float z2 = length / 2.0f;
		
		for (int i = 0;i < div;i++) {
			
			float x1 = FloatMath.cos((pi * 2.0f) * ((float)i / (float)div));
			float y1 = FloatMath.sin((pi * 2.0f) * ((float)i / (float)div));
			float x2 = FloatMath.cos((pi * 2.0f) * ((float)(i + 1) / (float)div));
			float y2 = FloatMath.sin((pi * 2.0f) * ((float)(i + 1) / (float)div));

			vertexList[i * 12] = r * x1;
			vertexList[i * 12 + 1] = r * y1;
			vertexList[i * 12 + 2] = z1;
			vertexList[i * 12 + 3] = r * x1;
			vertexList[i * 12 + 4] = r * y1;
			vertexList[i * 12 + 5] = z2;
			vertexList[i * 12 + 6] = r * x2;
			vertexList[i * 12 + 7] = r * y2;
			vertexList[i * 12 + 8] = z1;
			vertexList[i * 12 + 9] = r * x2;
			vertexList[i * 12 + 10] = r * y2;
			vertexList[i * 12 + 11] = z2;

			normalList[i * 12] = x1 * normalR;
			normalList[i * 12 + 1] = y1 * normalR;
			normalList[i * 12 + 2] = 0;
			normalList[i * 12 + 3] = x1 * normalR;
			normalList[i * 12 + 4] = y1 * normalR;
			normalList[i * 12 + 5] = 0;
			normalList[i * 12 + 6] = x2 * normalR;
			normalList[i * 12 + 7] = y2 * normalR;
			normalList[i * 12 + 8] = 0;
			normalList[i * 12 + 9] = x2 * normalR;
			normalList[i * 12 + 10] = y2 * normalR;
			normalList[i * 12 + 11] = 0;

			textureArray[i * 8] = (float)i / div;
			textureArray[i * 8 + 1] = 1;
			textureArray[i * 8 + 2] = (float)i / div;
			textureArray[i * 8 + 3] = 0;
			textureArray[i * 8 + 4] = (float)(i + 1) / div;
			textureArray[i * 8 + 5] = 1;
			textureArray[i * 8 + 6] = (float)(i + 1) / div;
			textureArray[i * 8 + 7] = 0;
			
		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLE_STRIP);
		
		return elm;
		
	}

	/**
	 * XZ平面に沿った円を底とする円柱（中空）elementを作成します。
	 * XZ平面に沿った円を底とする円柱（中空）をあらわすelementを作成します。
	 * @param r 底の半径
	 * @param length 円柱の長さ
	 * @param div 円の分割数
	 * @param normalR 円柱表面の向き（中心から見た法線。プラスなら表面が表、マイナスなら裏面が表）
	 * @return 作成された円柱element
	 */
	public static GbtkGL10Element createXZCylinderElement(float r, float length, int div, float normalR) {

		float[] vertexList = new float[div * 3 * 4];
		float[] normalList = new float[div * 3 * 4];
		float[] textureArray = new float[div * 4 * 2];

		float pi = (float)Math.PI;

		float y1 = -(length / 2.0f);
		float y2 = length / 2.0f;
		
		for (int i = 0;i < div;i++) {
			
			float x1 = FloatMath.cos((pi * 2.0f) * ((float)i / (float)div));
			float z1 = FloatMath.sin((pi * 2.0f) * ((float)i / (float)div));
			float x2 = FloatMath.cos((pi * 2.0f) * ((float)(i + 1) / (float)div));
			float z2 = FloatMath.sin((pi * 2.0f) * ((float)(i + 1) / (float)div));

			vertexList[i * 12] = r * x1;
			vertexList[i * 12 + 1] = y1;
			vertexList[i * 12 + 2] = r * z1;
			vertexList[i * 12 + 3] = r * x1;
			vertexList[i * 12 + 4] = y2;
			vertexList[i * 12 + 5] = r * z1;
			vertexList[i * 12 + 6] = r * x2;
			vertexList[i * 12 + 7] = y1;
			vertexList[i * 12 + 8] = r * z2;
			vertexList[i * 12 + 9] = r * x2;
			vertexList[i * 12 + 10] = y2;
			vertexList[i * 12 + 11] = r * z2;

			normalList[i * 12] = x1 * normalR;
			normalList[i * 12 + 1] = 0;
			normalList[i * 12 + 2] = z1 * normalR;
			normalList[i * 12 + 3] = x1 * normalR;
			normalList[i * 12 + 4] = 0;
			normalList[i * 12 + 5] = z1 * normalR;
			normalList[i * 12 + 6] = x2 * normalR;
			normalList[i * 12 + 7] = 0;
			normalList[i * 12 + 8] = z2 * normalR;
			normalList[i * 12 + 9] = x2 * normalR;
			normalList[i * 12 + 10] = 0;
			normalList[i * 12 + 11] = z2 * normalR;

			textureArray[i * 8] = 1.0f - (float)i / div;
			textureArray[1 + i * 8] = 1;
			textureArray[2 + i * 8] = 1.0f - (float)i / div;
			textureArray[3 + i * 8] = 0;
			textureArray[4 + i * 8] = 1.0f - (float)(i + 1) / div;
			textureArray[5 + i * 8] = 1;
			textureArray[6 + i * 8] = 1.0f - (float)(i + 1) / div;
			textureArray[7 + i * 8] = 0;
			
		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLE_STRIP);
		
		return elm;
		
	}

	/**
	 * YZ平面に沿った円を底とする円柱（中空）elementを作成します。
	 * YZ平面に沿った円を底とする円柱（中空）をあらわすelementを作成します。
	 * @param r 底の半径
	 * @param length 円柱の長さ
	 * @param div 円の分割数
	 * @param normalR 円柱表面の向き（中心から見た法線。プラスなら表面が表、マイナスなら裏面が表）
	 * @return 作成された円柱element
	 */
	public static GbtkGL10Element createYZCylinderElement(float r, float length, int div, float normalR) {

		float[] vertexList = new float[div * 3 * 4];
		float[] normalList = new float[div * 3 * 4];
		float[] textureArray = new float[div * 4 * 2];

		float pi = (float)Math.PI;

		float x1 = -(length / 2.0f);
		float x2 = length / 2.0f;
		
		for (int i = 0;i < div;i++) {
			
			float y1 = FloatMath.cos((pi * 2.0f) * ((float)i / (float)div));
			float z1 = FloatMath.sin((pi * 2.0f) * ((float)i / (float)div));
			float y2 = FloatMath.cos((pi * 2.0f) * ((float)(i + 1) / (float)div));
			float z2 = FloatMath.sin((pi * 2.0f) * ((float)(i + 1) / (float)div));

			vertexList[i * 12] = x1;
			vertexList[i * 12 + 1] = r * y1;
			vertexList[i * 12 + 2] = r * z1;
			vertexList[i * 12 + 3] = x2;
			vertexList[i * 12 + 4] = r * y1;
			vertexList[i * 12 + 5] = r * z1;
			vertexList[i * 12 + 6] = x1;
			vertexList[i * 12 + 7] = r * y2;
			vertexList[i * 12 + 8] = r * z2;
			vertexList[i * 12 + 9] = x2;
			vertexList[i * 12 + 10] = r * y2;
			vertexList[i * 12 + 11] = r * z2;

			normalList[i * 12] = 0;
			normalList[i * 12 + 1] = y1 * normalR;
			normalList[i * 12 + 2] = z1 * normalR;
			normalList[i * 12 + 3] = 0;
			normalList[i * 12 + 4] = y1 * normalR;
			normalList[i * 12 + 5] = z1 * normalR;
			normalList[i * 12 + 6] = 0;
			normalList[i * 12 + 7] = y2 * normalR;
			normalList[i * 12 + 8] = z2 * normalR;
			normalList[i * 12 + 9] = 0;
			normalList[i * 12 + 10] = y2 * normalR;
			normalList[i * 12 + 11] = z2 * normalR;

			textureArray[i * 8] = (float)i / div;
			textureArray[i * 8 + 1] = 0;
			textureArray[i * 8 + 2] = (float)i / div;
			textureArray[i * 8 + 3] = 1;
			textureArray[i * 8 + 4] = (float)(i + 1) / div;
			textureArray[i * 8 + 5] = 0;
			textureArray[i * 8 + 6] = (float)(i + 1) / div;
			textureArray[i * 8 + 7] = 1;

			textureArray[i * 8] = (float)i / div;
			textureArray[i * 8 + 1] = 1;
			textureArray[i * 8 + 2] = (float)i / div;
			textureArray[i * 8 + 3] = 0;
			
			textureArray[i * 8 + 4] = (float)(i + 1) / div;
			textureArray[i * 8 + 5] = 1;
			textureArray[i * 8 + 6] = (float)(i + 1) / div;
			textureArray[i * 8 + 7] = 0;
			
		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLE_STRIP);
		
		return elm;
		
	}
	
	/**
	 * X軸に沿って球体elementを作成します。
	 * X軸方向を緯線とする球体のelementを作成します。X軸に沿って巻きつける形でテクスチャを設定する場合に使用。
	 * @param r 半径
	 * @param div 頂点を作成する際の分割数
	 * @return 作成された球体element
	 */
	public static GbtkGL10Element createXSphereElement(float r, int div) {

		int i, j;

		float s1, s2, s3, s4, r1, r2, x1, x2, y1, y2, y3, y4, z1, z2, z3, z4;

		int vindex = 0;
		int nindex = 0;
		int tindex = 0;

		// 頂点数を算出
		int vertexNum = div * (6 + (div - 2) * 6);

		// 頂点データ格納用配列作成
		float[] vertexList = new float[vertexNum * 3];
		float[] normalList = new float[vertexNum * 3];
		float[] textureArray = new float[vertexNum * 2];

		// 球をDIV個の円筒（部分円錐）に分割
		for (i = 0;i < div;i++) {

			s1 = ((float)Math.PI * ((float)i / (float)div));
			s2 = ((float)Math.PI * ((float)(i + 1) / (float)div));

			r1 = FloatMath.sin(s1);
			r2 = FloatMath.sin(s2);
			
			x1 = (float)FloatMath.cos(s1);
			x2 = (float)FloatMath.cos(s2);

			// 個々の円筒（部分円錐）をdiv個に分割して頂点データを作成
			for (j = 0;j < div;j++) {

				s3 = ((float)Math.PI * ((float)j / (float)div) * 2.0f);
				s4 = ((float)Math.PI * ((float)(j + 1) / (float)div) * 2.0f);

				z1 = FloatMath.cos(s3) * r1;
				z2 = FloatMath.cos(s4) * r1;
				z3 = FloatMath.cos(s3) * r2;
				z4 = FloatMath.cos(s4) * r2;
				
				y1 = FloatMath.sin(s3) * r1;
				y2 = FloatMath.sin(s4) * r1;
				y3 = FloatMath.sin(s3) * r2;
				y4 = FloatMath.sin(s4) * r2;

				if (i == 0) {

					vertexList[vindex++] = 1.0f * r;
					vertexList[vindex++] = 0;
					vertexList[vindex++] = 0;
					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y3 * r;
					vertexList[vindex++] = z3 * r;
					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y4 * r;
					vertexList[vindex++] = z4 * r;

					normalList[nindex++] = 1.0f;
					normalList[nindex++] = 0;
					normalList[nindex++] = 0;
					normalList[nindex++] = x2;
					normalList[nindex++] = y3;
					normalList[nindex++] = z3;
					normalList[nindex++] = x2;
					normalList[nindex++] = y4;
					normalList[nindex++] = z4;

					textureArray[tindex++] = 1.0f;
					textureArray[tindex++] = 1.0f - (((float)(j + j + 1) / 2.0f) / div);
					textureArray[tindex++] = 1.0f - (float)(i + 1) / div;
					textureArray[tindex++] = 1.0f - ((float)j / div);
					textureArray[tindex++] = 1.0f - (float)(i + 1) / div;
					textureArray[tindex++] = 1.0f - ((float)(j + 1) / div);

				} else if(i == div - 1) {

					vertexList[vindex++] = x1 * r;
					vertexList[vindex++] = y1 * r;
					vertexList[vindex++] = z1 * r;
					vertexList[vindex++] = -1.0f * r;
					vertexList[vindex++] = 0;
					vertexList[vindex++] = 0;
					vertexList[vindex++] = x1 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z2 * r;

					normalList[nindex++] = x1;
					normalList[nindex++] = y1;
					normalList[nindex++] = z1;
					normalList[nindex++] = -1.0f;
					normalList[nindex++] = 0;
					normalList[nindex++] = 0;
					normalList[nindex++] = x1;
					normalList[nindex++] = y2;
					normalList[nindex++] = z2;

					textureArray[tindex++] = 1.0f - (float)i / div;
					textureArray[tindex++] = 1.0f - ((float)j / div);
					textureArray[tindex++] = 0.0f;
					textureArray[tindex++] = 1.0f - (((float)(j + j + 1) / 2.0f) / div);
					textureArray[tindex++] = 1.0f - (float)i / div;
					textureArray[tindex++] = 1.0f - ((float)(j + 1) / div);
					
				} else {
					
					vertexList[vindex++] = x1 * r;
					vertexList[vindex++] = y1 * r;
					vertexList[vindex++] = z1 * r;
					vertexList[vindex++] = x1 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z2 * r;
					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y3 * r;
					vertexList[vindex++] = z3 * r;

					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y3 * r;
					vertexList[vindex++] = z3 * r;
					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y4 * r;
					vertexList[vindex++] = z4 * r;
					vertexList[vindex++] = x1 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z2 * r;
				
					normalList[nindex++] = x1;
					normalList[nindex++] = y1;
					normalList[nindex++] = z1;
					normalList[nindex++] = x1;
					normalList[nindex++] = y2;
					normalList[nindex++] = z2;
					normalList[nindex++] = x2;
					normalList[nindex++] = y3;
					normalList[nindex++] = z3;

					normalList[nindex++] = x2;
					normalList[nindex++] = y3;
					normalList[nindex++] = z3;
					normalList[nindex++] = x2;
					normalList[nindex++] = y4;
					normalList[nindex++] = z4;
					normalList[nindex++] = x1;
					normalList[nindex++] = y2;
					normalList[nindex++] = z2;

					textureArray[tindex++] = 1.0f - (float)i / div;
					textureArray[tindex++] = 1.0f - ((float)j / div);
					textureArray[tindex++] = 1.0f - (float)i / div;
					textureArray[tindex++] = 1.0f - ((float)(j + 1) / div);
					textureArray[tindex++] = 1.0f - (float)(i + 1) / div;
					textureArray[tindex++] = 1.0f - ((float)j / div);
					textureArray[tindex++] = 1.0f - (float)(i + 1) / div;
					textureArray[tindex++] = 1.0f - ((float)j / div);
					textureArray[tindex++] = 1.0f - (float)(i + 1) / div;
					textureArray[tindex++] = 1.0f - ((float)(j + 1) / div);
					textureArray[tindex++] = 1.0f - (float)i / div;
					textureArray[tindex++] = 1.0f - ((float)(j + 1) / div);

				}
				
			}

		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLES);
		
		return elm;
		
	}

	/**
	 * Y軸に沿って球体elementを作成します。
	 * Y軸方向を緯線とする球体のelementを作成します。Y軸に沿って巻きつける形でテクスチャを設定する場合に使用。
	 * @param r 半径
	 * @param div 頂点を作成する際の分割数
	 * @return 作成された球体element
	 */
	public static GbtkGL10Element createYSphereElement(float r, int div) {

		int i, j;

		float s1, s2, s3, s4, r1, r2, x1, x2, x3, x4, y1, y2, z1, z2, z3, z4;

		int vindex = 0;
		int nindex = 0;
		int tindex = 0;

		// 頂点数を算出
		int vertexNum = div * (6 + (div - 2) * 6);

		// 頂点データ格納用配列作成
		float[] vertexList = new float[vertexNum * 3];
		float[] normalList = new float[vertexNum * 3];
		float[] textureArray = new float[vertexNum * 2];

		// 球をDIV個の円筒（部分円錐）に分割
		for (i = 0;i < div;i++) {

			s1 = ((float)Math.PI * ((float)i / (float)div));
			s2 = ((float)Math.PI * ((float)(i + 1) / (float)div));

			r1 = FloatMath.sin(s1);
			r2 = FloatMath.sin(s2);
			
			y1 = (float)FloatMath.cos(s1);
			y2 = (float)FloatMath.cos(s2);

			// 個々の円筒（部分円錐）をdiv個に分割して頂点データを作成
			for (j = 0;j < div;j++) {

				s3 = ((float)Math.PI * ((float)j / (float)div) * 2.0f);
				s4 = ((float)Math.PI * ((float)(j + 1) / (float)div) * 2.0f);

				z1 = FloatMath.cos(s3) * r1;
				z2 = FloatMath.cos(s4) * r1;
				z3 = FloatMath.cos(s3) * r2;
				z4 = FloatMath.cos(s4) * r2;
				
				x1 = FloatMath.sin(s3) * r1;
				x2 = FloatMath.sin(s4) * r1;
				x3 = FloatMath.sin(s3) * r2;
				x4 = FloatMath.sin(s4) * r2;

				if (i == 0) {

					vertexList[vindex++] = 0;
					vertexList[vindex++] = 1.0f * r;
					vertexList[vindex++] = 0;
					vertexList[vindex++] = x3 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z3 * r;
					vertexList[vindex++] = x4 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z4 * r;

					normalList[nindex++] = 0;
					normalList[nindex++] = 1.0f;
					normalList[nindex++] = 0;
					normalList[nindex++] = x3;
					normalList[nindex++] = y2;
					normalList[nindex++] = z3;
					normalList[nindex++] = x4;
					normalList[nindex++] = y2;
					normalList[nindex++] = z4;

					textureArray[tindex++] = (((float)(j + j + 1) / 2.0f) / div);
					textureArray[tindex++] = 0.0f;
					textureArray[tindex++] = ((float)j / div);
					textureArray[tindex++] = (float)(i + 1) / div;
					textureArray[tindex++] = ((float)(j + 1) / div);
					textureArray[tindex++] = (float)(i + 1) / div;

				} else if(i == div - 1) {

					vertexList[vindex++] = x1 * r;
					vertexList[vindex++] = y1 * r;
					vertexList[vindex++] = z1 * r;
					vertexList[vindex++] = 0;
					vertexList[vindex++] = -1.0f * r;
					vertexList[vindex++] = 0;
					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y1 * r;
					vertexList[vindex++] = z2 * r;

					normalList[nindex++] = x1;
					normalList[nindex++] = y1;
					normalList[nindex++] = z1;
					normalList[nindex++] = 0;
					normalList[nindex++] = -1.0f;
					normalList[nindex++] = 0;
					normalList[nindex++] = x2;
					normalList[nindex++] = y1;
					normalList[nindex++] = z2;

					textureArray[tindex++] = ((float)j / div);
					textureArray[tindex++] = (float)i / div;
					textureArray[tindex++] = (((float)(j + j + 1) / 2.0f) / div);
					textureArray[tindex++] = 1.0f;
					textureArray[tindex++] = ((float)(j + 1) / div);
					textureArray[tindex++] = (float)i / div;
					
				} else {
					
					vertexList[vindex++] = x1 * r;
					vertexList[vindex++] = y1 * r;
					vertexList[vindex++] = z1 * r;
					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y1 * r;
					vertexList[vindex++] = z2 * r;
					vertexList[vindex++] = x3 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z3 * r;

					vertexList[vindex++] = x3 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z3 * r;
					vertexList[vindex++] = x4 * r;
					vertexList[vindex++] = y2 * r;
					vertexList[vindex++] = z4 * r;
					vertexList[vindex++] = x2 * r;
					vertexList[vindex++] = y1 * r;
					vertexList[vindex++] = z2 * r;
				
					normalList[nindex++] = x1;
					normalList[nindex++] = y1;
					normalList[nindex++] = z1;
					normalList[nindex++] = x2;
					normalList[nindex++] = y1;
					normalList[nindex++] = z2;
					normalList[nindex++] = x3;
					normalList[nindex++] = y2;
					normalList[nindex++] = z3;

					normalList[nindex++] = x3;
					normalList[nindex++] = y2;
					normalList[nindex++] = z3;
					normalList[nindex++] = x4;
					normalList[nindex++] = y2;
					normalList[nindex++] = z4;
					normalList[nindex++] = x2;
					normalList[nindex++] = y1;
					normalList[nindex++] = z2;

					textureArray[tindex++] = ((float)j / div);
					textureArray[tindex++] = (float)i / div;
					textureArray[tindex++] = ((float)(j + 1) / div);
					textureArray[tindex++] = (float)i / div;
					textureArray[tindex++] = ((float)j / div);
					textureArray[tindex++] = (float)(i + 1) / div;
					textureArray[tindex++] = ((float)j / div);
					textureArray[tindex++] = (float)(i + 1) / div;
					textureArray[tindex++] = ((float)(j + 1) / div);
					textureArray[tindex++] = (float)(i + 1) / div;
					textureArray[tindex++] = ((float)(j + 1) / div);
					textureArray[tindex++] = (float)i / div;

				}
				
			}

		}

		GbtkGL10Element elm = new GbtkGL10Element();
		
		elm.setVertexArray(vertexList);
		elm.setNormalArray(normalList);
		elm.setTexturePointArray(textureArray);

		elm.setDrawType(DRAWTYPE_TRIANGLES);
		
		return elm;
		
	}
	
}
