package jp.programmingmat.www.gbtkgl10;

import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.Iterator;

import javax.microedition.khronos.opengles.GL10;

/**
 * 頂点（物体形状）データクラス。
 * elementに格納された頂点データの配列からAndroid内部で使用するFloatBufferを作成し、保持します。複数のelementを保持できますが、すべて同じmaterialが適用されます。
 */
public class GbtkGL10Geometry {

	private ArrayList<GbtkGL10Element> mElementList;

	private FloatBuffer mVertexBuffer;
	private FloatBuffer mNormalBuffer;
	private FloatBuffer mTexturePointBuffer;
	
	private int mVertexNum;

	/**
	 * 指定elementを物体形状として保持するgeometryオブジェクトを作成します。
	 * @param element
	 */
	public GbtkGL10Geometry(GbtkGL10Element element) {

		mElementList = new ArrayList<GbtkGL10Element>();
		
		mElementList.add(element);

		createBuffer();
		
	}

	/**
	 * 配列内のelementを物体形状として保持するgeometryオブジェクトを作成します。
	 * @param elements elementを格納した配列
	 */
	public GbtkGL10Geometry(GbtkGL10Element[] elements) {

		mElementList = new ArrayList<GbtkGL10Element>();
		
		for (int i = 0;i < elements.length;i++) {
			if (elements[i] != null) {
				mElementList.add(elements[i]);
			}
		}
		
		createBuffer();
		
	}

	/**
	 * 内部に作成したテクスチャバッファのFloatBufferを取得します。
	 * システムがOpenGL ESによるテクスチャ設定を行なう時に呼び出す関数です。
	 * @return テクスチャ座標を格納するFloatBuffer
	 */
	public FloatBuffer getTexturePointBuffer() {
		return mTexturePointBuffer;
	}

	/**
	 * 現在保持しているテクスチャ座標の配列に繰り返し数を乗じた上でFloatBuffer化します。
	 * システムが自動的に呼び出す関数です。
	 * @param udiv U方向のテクスチャ繰り返し数
	 * @param vdiv V方向のテクスチャ繰り返し数
	 */
	public void setTexturePointBuffer(float udiv, float vdiv) {

		Iterator<GbtkGL10Element> i = mElementList.iterator();

		float[] pointArray = new float[mVertexNum * 2];

		int offset = 0;
		
		while(i.hasNext()) {

			GbtkGL10Element elm = i.next();

			System.arraycopy(elm.getTexturePointArray(), 0, pointArray, offset, elm.getTexturePointArray().length);

			offset += elm.getVertexNum() * 2;
			
		}

		for (int n = 0;n < pointArray.length;n++) {
			if (n % 2 == 0) {
				pointArray[n] *= udiv;
			} else {
				pointArray[n] *= vdiv;
			}
		}
		
		mTexturePointBuffer = GbtkGL10Util.createFloatBuffer(pointArray);
	
	}

	/**
	 * 現在保持している頂点データの配列からOpenGL ESに渡すFloatBufferを作成します。
	 * システムから呼び出される関数です。
	 */
	private void createBuffer() {

		Iterator<GbtkGL10Element> i = mElementList.iterator();

		int vertexNum = 0;

		while(i.hasNext()) {
		
			GbtkGL10Element elm = i.next();
			
			elm.setDrawVertexOffset(vertexNum);
			
			vertexNum += elm.getVertexNum();

		}

		float[] vertexArray = new float[vertexNum * 3];
		float[] normalArray = new float[vertexNum * 3];

		i = mElementList.iterator();

		// 保持しているelementの頂点／法線配列を統合した配列を作成
		while(i.hasNext()) {

			GbtkGL10Element elm = i.next();

			System.arraycopy(elm.getVertexArray(), 0, vertexArray, elm.getDrawVertexOffset() * 3, elm.getVertexArray().length);
			System.arraycopy(elm.getNormalArray(), 0, normalArray, elm.getDrawVertexOffset() * 3, elm.getNormalArray().length);
			
		}

		// 統合した頂点／法線配列をFloatBufferに
		mVertexBuffer = GbtkGL10Util.createFloatBuffer(vertexArray);
		mNormalBuffer = GbtkGL10Util.createFloatBuffer(normalArray);

		mVertexNum = vertexNum;
		
	}

	/**
	 * XY平面に沿った円を底とする円柱（上下蓋つき／なし）elementを作成。
	 * XY平面に沿った円を底とする円柱をあらわすelementを作成します。
	 * @param r 底の半径
	 * @param length 円柱の長さ
	 * @param div 円の分割数
	 * @param open 上下に蓋（円）を追加するか。trueで追加。
	 * @return 作成された円柱element
	 */
	public static GbtkGL10Geometry createXYCylinderGeometry(float r, float length, int div, boolean open) {

		GbtkGL10Geometry geo;
		
		GbtkGL10Element cylinder = GbtkGL10Element.createXYCylinderElement(r, length, div, 1.0f);

		if (open) {
			geo = new GbtkGL10Geometry(cylinder);
		} else {

			GbtkGL10Element[] elements = new GbtkGL10Element[3];
			
			GbtkGL10Element circle1 = GbtkGL10Element.createXYCircleElement(r, div, 1);
			GbtkGL10Element circle2 = GbtkGL10Element.createXYCircleElement(r, div, -1);

			circle1.translate(0, 0, length / 2);
			circle2.translate(0, 0, -length / 2);
			
			elements[0] = circle1;
			elements[1] = circle2;
			elements[2] = cylinder;

			geo = new GbtkGL10Geometry(elements);
			
		}

		return geo;
		
	}

	/**
	 * XZ平面に沿った円を底とする円柱（上下蓋つき／なし）elementを作成。
	 * XZ平面に沿った円を底とする円柱をあらわすelementを作成します。
	 * @param r 底の半径
	 * @param length 円柱の長さ
	 * @param div 円の分割数
	 * @param open 上下に蓋（円）を追加するか。trueで追加。
	 * @return 作成された円柱element
	 */
	public static GbtkGL10Geometry createXZCylinderGeometry(float r, float length, int div, boolean open) {

		GbtkGL10Geometry geo;
		
		GbtkGL10Element cylinder = GbtkGL10Element.createXZCylinderElement(r, length, div, 1.0f);

		if (open) {
			geo = new GbtkGL10Geometry(cylinder);
		} else {

			GbtkGL10Element[] elements = new GbtkGL10Element[3];
			
			GbtkGL10Element circle1 = GbtkGL10Element.createXZCircleElement(r, div, 1);
			GbtkGL10Element circle2 = GbtkGL10Element.createXZCircleElement(r, div, -1);

			circle1.translate(0, length / 2, 0);
			circle2.translate(0, -length / 2, 0);
			
			elements[0] = circle1;
			elements[1] = circle2;
			elements[2] = cylinder;

			geo = new GbtkGL10Geometry(elements);
			
		}

		return geo;
		
	}

	/**
	 * YZ平面に沿った円を底とする円柱（上下蓋つき／なし）elementを作成。
	 * YZ平面に沿った円を底とする円柱をあらわすelementを作成します。
	 * @param r 底の半径
	 * @param length 円柱の長さ
	 * @param div 円の分割数
	 * @param open 上下に蓋（円）を追加するか。trueで追加。
	 * @return 作成された円柱element
	 */
	public static GbtkGL10Geometry createYZCylinderGeometry(float r, float length, int div, boolean open) {

		GbtkGL10Geometry geo;
		
		GbtkGL10Element cylinder = GbtkGL10Element.createYZCylinderElement(r, length, div, 1.0f);

		if (open) {
			geo = new GbtkGL10Geometry(cylinder);
		} else {

			GbtkGL10Element[] elements = new GbtkGL10Element[3];
			
			GbtkGL10Element circle1 = GbtkGL10Element.createYZCircleElement(r, div, 1);
			GbtkGL10Element circle2 = GbtkGL10Element.createYZCircleElement(r, div, -1);

			circle1.translate(length / 2, 0, 0);
			circle2.translate(-length / 2, 0, 0);
			
			elements[0] = circle1;
			elements[1] = circle2;
			elements[2] = cylinder;

			geo = new GbtkGL10Geometry(elements);
			
		}

		return geo;
		
	}

	/**
	 * 直方体geometryを作成。
	 * @param xSize X方向の大きさ
	 * @param ySize Y方向の大きさ
	 * @param zSize Z方向の大きさ
	 * @return 作成された直方体geometry
	 */
	public static GbtkGL10Geometry createBoxGeometry(float xSize, float ySize, float zSize) {

		GbtkGL10Element[] geoArray = new GbtkGL10Element[6];

		// X+面
		geoArray[0] = GbtkGL10Element.createYZPlaneElement(ySize, zSize, 1, xSize / 2.0f, 0, 0);

		// X-面
		geoArray[1] = GbtkGL10Element.createYZPlaneElement(ySize, zSize, -1, -xSize / 2.0f, 0, 0);

		// Y+面
		geoArray[2] = GbtkGL10Element.createXZPlaneElement(xSize, zSize, 1, 0, ySize / 2.0f, 0);

		// Y-面
		geoArray[3] = GbtkGL10Element.createXZPlaneElement(xSize, zSize, -1, 0, -ySize / 2.0f, 0);

		// Z+面
		geoArray[4] = GbtkGL10Element.createXYPlaneElement(xSize, ySize, 1, 0, 0, zSize / 2.0f);

		// Z-面
		geoArray[5] = GbtkGL10Element.createXYPlaneElement(xSize, ySize, -1, 0, 0, -zSize / 2.0f);
		
		GbtkGL10Geometry geo = new GbtkGL10Geometry(geoArray);
		
		return geo;
		
	}
	
	public void draw(GbtkGL10World world, GbtkGL10Material material) {

		if (mVertexBuffer == null) {
			return;
		}
		
		GL10 gl = world.getGLInterface();

		gl.glPushMatrix();
		
	    gl.glEnableClientState(GL10.GL_NORMAL_ARRAY);

		if (material != null) {
			material.putMaterial(world, this);
		}
	    
		if (mNormalBuffer != null) {
			gl.glNormalPointer(GL10.GL_FLOAT, 0, mNormalBuffer);
		}

		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		
    	gl.glVertexPointer(3, GL10.GL_FLOAT, 0, mVertexBuffer);

		Iterator<GbtkGL10Element> i = mElementList.iterator();

		while(i.hasNext()) {

			GbtkGL10Element elm = i.next();

			switch (elm.getDrawType()) {

			case GbtkGL10Element.DRAWTYPE_TRIANGLES:

				gl.glDrawArrays(GL10.GL_TRIANGLES, elm.getDrawVertexOffset(), elm.getVertexNum());

				break;
			
			case GbtkGL10Element.DRAWTYPE_TRIANGLE_STRIP:

				gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, elm.getDrawVertexOffset(), elm.getVertexNum());

				break;

			case GbtkGL10Element.DRAWTYPE_TRIANGLE_FAN:
			
				gl.glDrawArrays(GL10.GL_TRIANGLE_FAN, elm.getDrawVertexOffset(), elm.getVertexNum());

				break;
			
			}
			
		}

		gl.glPopMatrix();
		
	}
	
}
